/*
    This file is part of lorcon

    lorcon is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    lorcon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with lorcon; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Copyright (c) 2005 dragorn and Joshua Wright
*/

#ifndef __IEEE80211L_H__
#define __IEEE80211L_H__

#include "config.h"

#ifdef HAVE_STDINT_H
#include <stdint.h>
#endif
#ifdef HAVE_INTTYPES_H
#include <inttypes.h>
#endif

// Packet types and reason codes copied from various 
// sources (ieee docs and significant copying from the
// hostapd common defines)

#define BIT(x) (1 << (x))

#define WLAN_FC_TYPE_MGMT 0
#define WLAN_FC_TYPE_CTRL 1
#define WLAN_FC_TYPE_DATA 2

#define WLAN_FC_SUBTYPE_ASSOCREQ    0
#define WLAN_FC_SUBTYPE_ASSOCRESP   1
#define WLAN_FC_SUBTYPE_REASSOCREQ  2
#define WLAN_FC_SUBTYPE_REASSOCRESP 3
#define WLAN_FC_SUBTYPE_PROBEREQ    4
#define WLAN_FC_SUBTYPE_PROBERESP   5
#define WLAN_FC_SUBTYPE_BEACON      8
#define WLAN_FC_SUBTYPE_ATIM        9
#define WLAN_FC_SUBTYPE_DISASSOC    10
#define WLAN_FC_SUBTYPE_AUTH        11
#define WLAN_FC_SUBTYPE_DEAUTH      12

#define WLAN_FC_SUBTYPE_PSPOLL      10
#define WLAN_FC_SUBTYPE_RTS         11
#define WLAN_FC_SUBTYPE_CTS         12
#define WLAN_FC_SUBTYPE_ACK         13
#define WLAN_FC_SUBTYPE_CFEND       14
#define WLAN_FC_SUBTYPE_CFENDACK    15

#define WLAN_FC_SUBTYPE_DATA        0
#define WLAN_FC_SUBTYPE_DATACFACK   1
#define WLAN_FC_SUBTYPE_DATACFPOLL  2
#define WLAN_FC_SUBTYPE_DATACFACKPOLL 3
#define WLAN_FC_SUBTYPE_DATANULL    4
#define WLAN_FC_SUBTYPE_CFACK       5
#define WLAN_FC_SUBTYPE_CFACKPOLL   6

// Framecontrol bits
#define WLAN_FC_PVER                (BIT(1) | BIT(0))
#define WLAN_FC_TODS                BIT(8)
#define WLAN_FC_FROMDS              BIT(9)
#define WLAN_FC_MOREFRAG            BIT(10)
#define WLAN_FC_RETRY               BIT(11)
#define WLAN_FC_PWRMGT              BIT(12)
#define WLAN_FC_MOREDATA            BIT(13)
#define WLAN_FC_ISWEP               BIT(14)
#define WLAN_FC_ORDER               BIT(15)

// Auth stuff
#define WLAN_AUTH_OPEN              0
#define WLAN_AUTH_SHARED_KEY        1
#define WLAN_AUTH_CHALLENGE_LEN     128

// 802.11 capabilities
#define WLAN_CAPABILITY_ESS         BIT(0)
#define WLAN_CAPABILITY_IBSS        BIT(1)
#define WLAN_CAPABILITY_CF_POLLABLE BIT(2)
#define WLAN_CAPABILITY_CF_POLL_REQUEST BIT(3)
#define WLAN_CAPABILITY_PRIVACY     BIT(4)

// Reason codes
#define WLAN_REASON_UNSPECIFIED                     1
#define WLAN_REASON_PREV_AUTH_NOT_VALID             2
#define WLAN_REASON_DEAUTH_LEAVING                  3
#define WLAN_REASON_DISASSOC_DUE_TO_INACTIVITY      4
#define WLAN_REASON_DISASSOC_AP_BUSY                5
#define WLAN_REASON_CLASS2_FRAME_FROM_NONAUTH_STA   6
#define WLAN_REASON_CLASS3_FRAME_FROM_NONASSOC_STA  7
#define WLAN_REASON_DISASSOC_STA_HAS_LEFT           8
#define WLAN_REASON_STA_REQ_ASSOC_WITHOUT_AUTH      9

// Status codes
#define WLAN_STATUS_SUCCESS                     0
#define WLAN_STATUS_UNSPECIFIED_FAILURE         1
#define WLAN_STATUS_CAPS_UNSUPPORTED            10
#define WLAN_STATUS_REASSOC_NO_ASSOC            11
#define WLAN_STATUS_ASSOC_DENIED_UNSPEC         12
#define WLAN_STATUS_NOT_SUPPORTED_AUTH_ALG      13
#define WLAN_STATUS_UNKNOWN_AUTH_TRANSACTION    14
#define WLAN_STATUS_CHALLENGE_FAIL              15
#define WLAN_STATUS_AUTH_TIMEOUT                16
#define WLAN_STATUS_AP_UNABLE_TO_HANDLE_NEW_STA 17
#define WLAN_STATUS_ASSOC_DENIED_RATES          18

#define WLAN_STATUS_ASSOC_DENIED_NOSHORT        19
#define WLAN_STATUS_ASSOC_DENIED_NOPBCC         20
#define WLAN_STATUS_ASSOC_DENIED_NOAGILITY      21

#define WLAN_STATUS_INVALID_IE                  40
#define WLAN_STATUS_GROUP_CIPHER_NOT_VALID      41
#define WLAN_STATUS_PAIRWISE_CIPHER_NOT_VALID   42
#define WLAN_STATUS_AKMP_NOT_VALID              43
#define WLAN_STATUS_UNSUPPORTED_RSN_IE_VERSION  44
#define WLAN_STATUS_INVALID_RSN_IE_CAPAB        45
#define WLAN_STATUS_CIPHER_REJECTED_PER_POLICY  46

// Tagged parameters
#define WLAN_TAGPARM_SSID           0
#define WLAN_TAGPARM_SUPPRATES      1
#define WLAN_TAGPARM_FH_PARAMS      2
#define WLAN_TAGPARM_DS_PARAMS      3
#define WLAN_TAGPARM_CF_PARAMS      4
#define WLAN_TAGPARM_TIM            5
#define WLAN_TAGPARM_IBSS_PARAMS    6
#define WLAN_TAGPARM_CHALLENGE      16
#define WLAN_TAGPARM_GENERIC        221

// 802.11 management frames
// stolen from hostapd
#define IEEE80211_HDRLEN (sizeof(struct ieee80211_hdr))

typedef struct ieee80211_hdr {
	uint16_t frame_control;
	uint16_t duration;
	uint8_t addr1[6];
	uint8_t addr2[6];
	uint8_t addr3[6];
	uint16_t seq_ctrl;
	/* followed by 'u8 addr4[6];' if ToDS and FromDS is set in data frame
	 */
} __attribute__ ((packed));

typedef struct ieee80211_mgmt {
    uint16_t frame_control;
    uint16_t duration;
    uint8_t da[6];
    uint8_t sa[6];
    uint8_t bssid[6];
    uint16_t seq_ctrl;
    union {
        struct {
            uint16_t auth_alg;
            uint16_t auth_transaction;
            uint16_t status_code;
            /* possibly followed by Challenge text */
            uint8_t variable[0];
        } __attribute__ ((packed)) auth;
        struct {
            uint16_t reason_code;
        } __attribute__ ((packed)) deauth;
        struct {
            uint16_t capab_info;
            uint16_t listen_interval;
            /* followed by SSID and Supported rates */
            uint8_t variable[0];
        } __attribute__ ((packed)) assoc_req;
        struct {
            uint16_t capab_info;
            uint16_t status_code;
            uint16_t aid;
            /* followed by Supported rates */
            uint8_t variable[0];
        } __attribute__ ((packed)) assoc_resp, reassoc_resp;
        struct {
            uint16_t capab_info;
            uint16_t listen_interval;
            uint8_t current_ap[6];
            /* followed by SSID and Supported rates */
            uint8_t variable[0];
        } __attribute__ ((packed)) reassoc_req;
        struct {
            uint16_t reason_code;
        } __attribute__ ((packed)) disassoc;
        struct {
            uint8_t variable[0];
        } __attribute__ ((packed)) probe_req;
        struct {
            uint8_t timestamp[8];
            uint16_t beacon_int;
            uint16_t capab_info;
            /* followed by some of SSID, Supported rates,
             * FH Params, DS Params, CF Params, IBSS Params, TIM */
            uint8_t variable[0];
        } __attribute__ ((packed)) beacon;
    } u;
} __attribute__ ((packed));

/* IEEE 802.11 fixed parameters */
typedef struct iee80211_fixparm {
    uint8_t timestamp[8];
    uint16_t beaconinterval;
    uint16_t capability;
} __attribute__ ((packed));


#endif

