/*
    This file is part of lorcon

    lorcon is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    lorcon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with lorcon; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Copyright (c) 2005 dragorn and Joshua Wright
*/

#include "config.h"

#include <stdint.h>
#include <errno.h>
#include <string.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>

#include "tx80211.h"
#include "tx80211_errno.h"
#include "wginject.h"
#include "ajinject.h"
#include "p54inject.h"

struct tx80211_cardlist *tx80211_getcardlist() {
	struct tx80211_cardlist *ret = NULL;

	ret = (struct tx80211_cardlist *) malloc(sizeof(struct tx80211_cardlist));
	if (ret == NULL) {
		return NULL;
	}

	ret->cardnames = (char **) malloc(sizeof(char *) * INJ_MAX);
	if (ret->cardnames == NULL) {
		free(ret);
		return NULL;
	}

	ret->descriptions = (char **) malloc(sizeof(char *) * INJ_MAX);
	if (ret->descriptions == NULL) {
		free(ret->cardnames);
		free(ret);
		return NULL;
	}

	ret->capabilities = (int *) malloc(sizeof(int) * INJ_MAX);
	if (ret->capabilities == NULL) {
		free(ret->cardnames);
		free(ret->descriptions);
		free(ret);
		return NULL;
	}

	ret->num_cards = 0;

	/* Too bad there isn't a good way to automate this, fortunately we don't
	 * edit this list all that often.  */

	ret->cardnames[INJ_NODRIVER] = strdup("nodriver");
	ret->descriptions[INJ_NODRIVER] = strdup("No such driver (placeholder)");
	ret->capabilities[INJ_NODRIVER] = 0;
	ret->num_cards++;

	ret->cardnames[INJ_WLANNG] = strdup("wlan-ng");
	ret->descriptions[INJ_WLANNG] = strdup("Wlan-ng Prism2 802.11b (patched)");
	ret->capabilities[INJ_WLANNG] = tx80211_wlanng_capabilities();
	ret->num_cards++;

	ret->cardnames[INJ_HOSTAP] = strdup("hostap");
	ret->descriptions[INJ_HOSTAP] = strdup("HostAP Prism2 802.11b");
	ret->capabilities[INJ_HOSTAP] = tx80211_hostap_capabilities();
	ret->num_cards++;

	ret->cardnames[INJ_AIRJACK] = strdup("airjack");
	ret->descriptions[INJ_AIRJACK] = strdup("AirJack Prism2 802.11b (defunct)");
	ret->capabilities[INJ_AIRJACK] = tx80211_airjack_capabilities();
	ret->num_cards++;
	
	ret->cardnames[INJ_PRISM54] = strdup("prism54");
	ret->descriptions[INJ_PRISM54] = strdup("Prism54 PrismGT 802.11b/g");
	ret->capabilities[INJ_PRISM54] = tx80211_prism54_capabilities();
	ret->num_cards++;

	ret->cardnames[INJ_MADWIFI] = strdup("madwifi");
	ret->descriptions[INJ_MADWIFI] = strdup("MadWifi Atheros 802.11a/b/g");
	ret->capabilities[INJ_MADWIFI] = tx80211_madwifi_capabilities();
	ret->num_cards++;

	ret->cardnames[INJ_RTL8180] = strdup("rtl8180");
	ret->descriptions[INJ_RTL8180] = strdup("RealTek LAN 8180 802.11b");
	ret->capabilities[INJ_RTL8180] = tx80211_rtl8180_capabilities();
	ret->num_cards++;

	ret->cardnames[INJ_RT2500] = strdup("rt2500");
	ret->descriptions[INJ_RT2500] = strdup("Raylink 2500 802.11b");
	ret->capabilities[INJ_RT2500] = tx80211_rt25kinject_capabilities();
	ret->num_cards++;

	return ret;
}

void tx80211_freecardlist(struct tx80211_cardlist *in_list) {
	int x;

	/* Free the string lists */
	for (x = 0; x < in_list->num_cards; x++) {
		free(in_list->cardnames[x]);
		free(in_list->descriptions[x]);
	}
	
	free(in_list->cardnames);
	free(in_list->descriptions);
	free(in_list->capabilities);
	free(in_list);
}


int tx80211_resolvecard(const char *in_str) {

	if (!strcasecmp(in_str, "wlan-ng") || !strcasecmp(in_str, "wlanng"))
		return INJ_WLANNG;

	if (!strcasecmp(in_str, "host-ap") || !strcasecmp(in_str, "hostap"))
		return INJ_HOSTAP;

	if (!strcasecmp(in_str, "airjack"))
		return INJ_AIRJACK;

	if (!strcasecmp(in_str, "prism54") || !strcasecmp(in_str, "prismgt"))
		return INJ_PRISM54;

	if (!strcasecmp(in_str, "madwifi"))
		return INJ_MADWIFI;

	if (!strcasecmp(in_str, "rtl8180") || !strcasecmp(in_str, "rt8180"))
		return INJ_RTL8180;

	if (!strcasecmp(in_str, "rt2500") || !strcasecmp(in_str, "rtl2500") ||
        !strcasecmp(in_str, "rt25k")  || !strcasecmp(in_str, "rtl25k"))
		return INJ_RT2500;

	return INJ_NODRIVER;
}

int tx80211_init(struct tx80211 *in_tx, char *in_ifname, int in_injector) {
	/* Zero the struct entirely */
	memset(in_tx, 0, sizeof(struct tx80211));

	strncpy(in_tx->ifname, in_ifname, MAX_IFNAME_LEN);

	switch(in_injector) {
		case INJ_WLANNG:
			tx80211_wlanng_init(in_tx);
			break;

        case INJ_AIRJACK:
            tx80211_airjack_init(in_tx);
            break;

        case INJ_PRISM54:
            tx80211_prism54_init(in_tx);
            break;

        case INJ_MADWIFI:
            tx80211_madwifi_init(in_tx);
            break;

        case INJ_HOSTAP:
            tx80211_hostap_init(in_tx);
            break;

		default:
			return TX80211_ENOSUCHINJ;
			break;
	}

	return TX80211_ENOERR;
}

int tx80211_getcapabilities(struct tx80211 *in_tx) {
	return in_tx->capabilities;
}

int tx80211_open(struct tx80211 *in_tx) {
	if (in_tx->open_callthrough == NULL)
		return TX80211_ENOHANDLER;

	return (in_tx->open_callthrough)(in_tx);
}

int tx80211_close(struct tx80211 *in_tx) {
	if (in_tx->close_callthrough == NULL)
		return TX80211_ENOHANDLER;

	return (in_tx->close_callthrough)(in_tx);
}

int tx80211_setmode(struct tx80211 *in_tx, int in_mode) {
	if (in_tx->setmode_callthrough == NULL)
		return TX80211_ENOHANDLER;

	return (in_tx->setmode_callthrough)(in_tx, in_mode);
}

int tx80211_getmode(struct tx80211 *in_tx) {
	if (in_tx->getmode_callthrough == NULL)
		return TX80211_ENOHANDLER;

	return (in_tx->getmode_callthrough)(in_tx);
}

int tx80211_setfunctionalmode(struct tx80211 *in_tx, int in_fmode) {
	if (in_tx->setfuncmode_callthrough == NULL)
		return TX80211_ENOHANDLER;

	return (in_tx->setfuncmode_callthrough)(in_tx, in_fmode);
}

int tx80211_setchannel(struct tx80211 *in_tx, int in_chan) {
	if (in_tx->setchan_callthrough == NULL)
		return TX80211_ENOHANDLER;

	return (in_tx->setchan_callthrough)(in_tx, in_chan);
}

int tx80211_getchan(struct tx80211 *in_tx) {
	if (in_tx->getchan_callthrough == NULL)
		return TX80211_ENOHANDLER;

	return (in_tx->getchan_callthrough)(in_tx);
}

int tx80211_txpacket(struct tx80211 *in_tx, struct tx80211_packet *in_packet) {
	if (in_tx->txpacket_callthrough == NULL)
		return TX80211_ENOHANDLER;

	return (in_tx->txpacket_callthrough)(in_tx, in_packet);
}


